--- dovecot-1.1.1-orig/src/lib/mountpoint.c	2008-06-12 08:45:10.000000000 +0200
+++ dovecot-1.1.1/src/lib/mountpoint.c	2008-07-05 18:04:11.000000000 +0200
@@ -5,7 +5,10 @@
 
 #include <sys/stat.h>
 
-#ifdef HAVE_STATVFS_MNTFROMNAME
+#ifdef HAVE_VMOUNT_H
+#  include <stdio.h>
+#  include <sys/vmount.h> /* AIX */
+#elif defined(HAVE_STATVFS_MNTFROMNAME)
 #  include <sys/statvfs.h> /* NetBSD 3.0+, FreeBSD 5.0+ */
 #  define STATVFS_STR "statvfs"
 #elif defined(HAVE_STATFS_MNTFROMNAME)
@@ -37,6 +40,12 @@
 #ifndef MNTTYPE_IGNORE
 #  define MNTTYPE_IGNORE "ignore"
 #endif
+#ifndef MNTTYPE_JFS
+#  define MNTTYPE_JFS "jfs"
+#endif
+#ifndef MNTTYPE_NFS
+#  define MNTTYPE_NFS "nfs"
+#endif
 
 
 int mountpoint_get(const char *path, pool_t pool, struct mountpoint *point_r)
@@ -93,7 +102,64 @@
 	}
 	block_size = st.st_blksize;
 
-#ifdef HAVE_SYS_MNTTAB_H
+#ifdef HAVE_VMOUNT_H
+{
+	char *mtab = NULL;
+	int i, count, size;
+	struct vmount *vmt;
+
+	count = mntctl(MCTL_QUERY, sizeof(size), (char *)&size);
+	if (count != 0) {
+		i_error("mntctl failed with %i!", count);
+		return -1;
+	}
+	mtab  = p_new(pool, char, size);
+	count = mntctl(MCTL_QUERY, size, mtab);
+	if (count == 0) {
+		i_error("mntctl failed, size has changed!");
+		return -1;
+	}
+
+	for (i = 0; i < count; i++) {
+
+		struct stat vst;
+		char *vmt_object, *vmt_stub, *vmt_hostname;
+
+		vmt = (struct vmount *)(i == 0 ? mtab : (((char *)vmt) + vmt->vmt_length));
+		vmt_hostname = ((char *)vmt) + vmt->vmt_data[VMT_HOSTNAME].vmt_off;
+		vmt_object   = ((char *)vmt) + vmt->vmt_data[VMT_OBJECT].vmt_off;
+		vmt_stub     = ((char *)vmt) + vmt->vmt_data[VMT_STUB].vmt_off;
+
+		if (stat(vmt_stub,&vst) < 0 || st.st_dev != vst.st_dev)
+			continue;
+
+		switch (vmt->vmt_gfstype) {
+
+			case MNT_NFS:
+			case MNT_NFS3:
+			case MNT_NFS4:
+			case MNT_RFS4:
+				device_path = t_strconcat(vmt_hostname, ":", vmt_object, NULL);
+				mount_path  = vmt_stub;
+				type        = MNTTYPE_NFS;
+				break;
+
+			case MNT_J2:
+			case MNT_JFS:
+				device_path = vmt_object;
+				mount_path  = vmt_stub;
+				type        = MNTTYPE_JFS;
+				break;
+		}
+		break;
+	}
+	if (device_path == NULL) {
+		p_free(pool, mtab);
+		mtab = NULL;
+	}
+}
+#elif defined(HAVE_SYS_MNTTAB_H)
+
 	/* Solaris */
 	f = fopen(MTAB_PATH, "r");
 	if (f == NULL) {
@@ -146,6 +212,11 @@
 	point_r->mount_path = p_strdup(pool, mount_path);
 	point_r->type = p_strdup(pool, type);
 	point_r->block_size = block_size;
+
+#ifdef HAVE_VMOUNT_H
+	if (mtab != NULL)
+		p_free(pool, mtab);
+#endif
 	return 1;
 #endif
 }
